package de.uni_passau.spl.featurebite.adapter.detector;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.FieldNode;

import de.uni_passau.spl.featurebite.stubs.StubUtil;

/**
 * The {@code FieldAdapter} is a common {@link FieldVisitor}. It just identifies
 * whether a visited field has a stub annotation or not.
 * 
 * @author Claus Hunsen
 */
public class FieldAdapter extends FieldVisitor {

	/**
	 * The representing {@link FieldNode} for the field examined using this
	 * {@link FieldAdapter} object.
	 * 
	 * NOTE: Only used for extracting {@link FieldNode#name}!
	 */
	 private final FieldNode fn;

	/**
	 * Flag whether the examined field is a stub or not.
	 * 
	 * @see #visitAnnotation(String, boolean)
	 */
	private boolean isStubField;

	/**
	 * Constructor that accepts the field visitor that is higher in visitor
	 * chain and a field node representing the method within the field visitor.
	 * 
	 * @param mv
	 *            the {@link FieldVisitor} that is passed by ASM's visitor
	 *            calling chain
	 * @param mn
	 *            the {@link FieldNode} representive of the examined field
	 */
	public FieldAdapter(FieldVisitor fv, FieldNode fn) {
		super(Opcodes.ASM4, fv);
		this.fn = fn;

		this.isStubField = false;
	}

	/**
	 * Visits the current annotation and checks if it is one that identifies
	 * stubs. Does not remove the annotation!
	 * 
	 * @see StubDetector#visitAnnotation(String, boolean)
	 * @see MethodVisitor#visitAnnotation(String, boolean)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(String desc, boolean visible) {

		if (StubUtil.isStubAnnotation(desc)) {
			this.isStubField = true;
			/* remove annotation with: return null; */
		}

		return super.visitAnnotation(desc, visible);
	}

	public FieldNode getFieldNode() {
		return fn;
	}

	/**
	 * TODO doc
	 */
	public boolean isStubField() {
		return this.isStubField;
	}

}
