package de.uni_passau.spl.featurebite.adapter.misc;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

/**
 * A {@link Printer} is a {@link ClassVisitor} that prints a simple class trace
 * to {@link System#out}.
 * 
 * @author Claus Hunsen
 */
public class Printer extends ClassVisitor {

	public Printer() {
		super(Opcodes.ASM4);
	}

	@Override
	public void visit(int version, int access, String name, String signature, String superName,
			String[] interfaces) {
		System.out.print(name + " extends " + superName);
		if (interfaces.length > 0) {
			System.out.print(" implements ");
			for (int i = 0; i < interfaces.length; i++) {
				System.out.print(interfaces[i] + ", ");
			}
		}
		if (signature != null) {
			System.out.print(" // " + signature);
		}
		System.out.println(" {");
	}

	@Override
	public void visitSource(String source, String debug) {
	}

	@Override
	public void visitOuterClass(String owner, String name, String desc) {
	}

	@Override
	public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
		return null;
	}

	@Override
	public void visitAttribute(Attribute attr) {
	}

	@Override
	public void visitInnerClass(String name, String outerName, String innerName, int access) {
		System.out.println("    class " + name);
	}

	@Override
	public FieldVisitor visitField(int access, String name, String desc, String signature,
			Object value) {
		System.out.print("    " + desc + " " + name);
		if (signature != null) {
			System.out.print(" //" + signature);
		}
		System.out.println();
		return null;
	}

	@Override
	public MethodVisitor visitMethod(int access, String name, String desc, String signature,
			String[] exceptions) {
		System.out.print("    " + name + " " + desc);
		if (signature != null) {
			System.out.print(" //" + signature);
		}
		System.out.println();
		return null;
	}

	@Override
	public void visitEnd() {
		System.out.println("}");
	}

}