package de.uni_passau.spl.featurebite.adapter.rename;

import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.commons.InstructionAdapter;

/**
 * The {@link SuperClassRenameAdapter} can make renaming of an super call within
 * a constructor, when a super class has changed. You provide a
 * {@link RenameUtil} that holds the renaming pairs (old -> new), and it
 * substitutes accordingly as it goes.
 * 
 * @author Claus Hunsen
 */
public class SuperClassRenameAdapter extends InstructionAdapter {

	/**
	 * The utility that handles the renamings.
	 */
	private final RenameUtil ru;

	/**
	 * Creates a new adapter that decorates the given other visitor, and renames
	 * the super call using the given {@link RenameUtil}.
	 * 
	 * @param mv
	 *            the class visitor to decorate with renaming capability
	 * @param ru
	 *            the {@link RenameUtil} that handles any renaming action
	 */
	public SuperClassRenameAdapter(MethodVisitor mv, RenameUtil ru) {
		super(mv);
		this.ru = ru;
	}

	/**
	 * Renames the super call to the new super class name.
	 * 
	 * @see InstructionAdapter#invokespecial(String, String, String)
	 */
	@Override
	public void invokespecial(String owner, String name, String desc) {
		super.invokespecial(this.ru.rename(owner), name, desc);
	}

}
