package de.uni_passau.spl.featurebite.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.tree.ClassNode;

public class ClassNodeTransformer {

	private ClassNodeTransformer() {
	} 

	/**
	 * Opens the given class file with ASM and returns the result as a
	 * {@link ClassNode}.
	 * 
	 * @param classFile
	 *            the .class file to be opened and prepared
	 * 
	 * @return a {@link ClassNode} instance representing the .class file given
	 *         through the parameters
	 * 
	 * @throws IOException
	 *             if the class file could not be opened, altered or copied
	 * @throws FileNotFoundException
	 *             if the class file could not be found
	 */
	public static ClassNode createClassNodeFrom(File classFile)
			throws IOException, FileNotFoundException {

		/* read class file */
		FileInputStream baseFileStream = new FileInputStream(classFile);
		ClassReader baseReader = new ClassReader(baseFileStream);
		baseFileStream.close();
		/* create new class node and put class file code in it */
		ClassNode baseNode = new ClassNode();
		baseReader.accept(baseNode, ClassReader.SKIP_DEBUG);

		return baseNode;
	}

	/**
	 * Retrieves the class as a {@link ClassNode} from the given
	 * {@link ClassWriter} object.<br />
	 * 
	 * @return the {@link ClassNode} representing the given {@link ClassWriter}
	 */
	public static ClassNode createClassNodeFrom(ClassWriter cw) {
		// read class from classwriter
		ClassReader crMerged = new ClassReader(cw.toByteArray());
		// create new class node for it ...
		ClassNode newBaseNode = new ClassNode();
		// ... and put the read class into it
		crMerged.accept(newBaseNode, ClassReader.SKIP_DEBUG);

		return newBaseNode;
	}

	/**
	 * Retrieves the class as a {@link ClassWriter} from the given
	 * {@link ClassNode} object.<br />
	 * 
	 * @return the {@link ClassWriter} representing the given {@link ClassNode}
	 */
	public static ClassWriter createClassWriterFrom(ClassNode cn) {
		ClassWriter cw = new ClassWriter(0);
		cn.accept(cw);

		return cw;
	}

	/**
	 * Retrieves the class as a {@link ClassReader} from the given
	 * {@link ClassNode} object.<br />
	 * 
	 * @return the {@link ClassReader} representing the given {@link ClassNode}
	 */
	public static ClassReader createClassReaderFrom(ClassNode cn) {
		ClassWriter cw = createClassWriterFrom(cn);
		return new ClassReader(cw.toByteArray());
	}

}
