/**
 * 
 */
package de.uni_passau.spl.featurebite.util;

import org.apache.commons.io.FilenameUtils;

/**
 * The {@link FilePathUtils} delegates methods to the {@link FilenameUtils}, but
 * uses sometimes other, more understandable and for the context more
 * appropriate names for them.
 * 
 * @author Claus Hunsen
 * 
 */
public class FilePathUtils {

	/**
	 * Private constructor.
	 */
	private FilePathUtils() {
	}

	/**
	 * Changes any separator to a forward slash ('/') as in Java package paths.
	 * 
	 * @param path
	 *            the path whose separators need to be changed
	 * @return the path with new separators
	 * 
	 * @see FilenameUtils#separatorsToUnix(String)
	 */
	public static String separatorsToPackageStyle(String path) {
		return FilenameUtils.separatorsToUnix(path);
	}

	/**
	 * Changes any separator to the system one, so that the path can be used
	 * properly as file path.
	 * 
	 * @param path
	 *            the path whose separators need to be changed
	 * @return the path with system separators
	 * 
	 * @see FilenameUtils#separatorsToSystem(String)
	 */
	public static String separatorsToSystemStyle(String path) {
		return FilenameUtils.separatorsToSystem(path);
	}

	/**
	 * Removes the extension from the path.
	 * 
	 * @param path
	 *            the file with extension
	 * @return the file path without extension
	 * 
	 * @see FilenameUtils#removeExtension(String)
	 */
	public static String removeExtension(String path) {
		return FilenameUtils.removeExtension(path);
	}

	/**
	 * Gets the extension from the given file path.
	 * 
	 * @param path
	 *            the file path with extension
	 * @return the extension
	 * 
	 * @see FilenameUtils#getExtension(String)
	 */
	public static String getExtension(String path) {
		return FilenameUtils.getExtension(path);
	}

	/**
	 * Gets the name of the given file.
	 * 
	 * @param path
	 *            the file's path
	 * @return its name
	 * 
	 * @see FilenameUtils#getBaseName(String)
	 */
	public static String getFileName(String path) {
		return FilenameUtils.getBaseName(path);
	}

	/**
	 * Concatenes paths. For further rules, see
	 * {@link FilenameUtils#concat(String, String)}.
	 * 
	 * @param basedir
	 *            the beginning of the path
	 * @param sub
	 *            the subfolder or file that is concatenated to the basedir
	 * @return the concatenated path
	 * 
	 * @see FilenameUtils#concat(String, String)
	 */
	public static String concatPaths(String basedir, String sub) {
		return FilenameUtils.concat(basedir, sub);
	}

}
