package de.uni_passau.spl.featurebite.util;

/**
 * The {@link NameMangling} utility constructs mangled unique names from given
 * field or method names and descriptions.
 * 
 * @author Claus Hunsen
 */
public class NameMangling {

	/**
	 * The format for field mangling.
	 */
	private static final String FIELD_FORMAT = "%s+%s";

	/**
	 * The format for method mangling.
	 */
	private static final String METHOD_FORMAT = "%s#%s";

	/* util class does not need constructor */
	private NameMangling() {
	}

	/**
	 * Returns the mangled name of the given field.
	 * 
	 * @param fieldName
	 *            the field's name
	 * @param fieldDesc
	 *            the field's description (type)
	 * @return the mangled unique name of the field
	 */
	public static String getFieldIdentifier(final String fieldName, final String fieldDesc) {
		return String.format(FIELD_FORMAT, fieldName, fieldDesc);
	}

	/**
	 * Returns the mangled name of the given method.
	 * 
	 * @param methodName
	 *            the method's name
	 * @param methodDesc
	 *            the method's description (parameter types + return type)
	 * @return the mangled unique name of the method
	 */
	public static String getMethodIdentifier(final String methodName, final String methodDesc) {
		return String.format(METHOD_FORMAT, methodName, methodDesc);
	}

}
