package de.uni_passau.spl.featurebite.verification;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.tree.ClassNode;

/**
 * TODO
 * 
 * @author Claus Hunsen
 * 
 */
public class VerificationClassLoader extends ClassLoader {

	private final Map<String, Class<?>> loadedClasses = new HashMap<String, Class<?>>();

	private final Map<String, ClassNode> classNodes = new HashMap<String, ClassNode>();

	public VerificationClassLoader(Set<ClassNode> classNodes, ClassLoader cp) {
		super(cp);

		for (ClassNode node : classNodes) {
			this.classNodes.put(node.name.replace('/', '.'), node);
		}
		// IMPORTANT: this need to be separate!
		for (ClassNode node : classNodes) {
			this.addComposedClassToClassLoader(node);
		}
	}

	private final Class<?> defineComposedClass(String name, byte[] b, int off,
			int len) throws ClassFormatError {
		return defineClass(name, b, off, len, null);
	}

	/**
	 * TODO doku
	 * 
	 * @param baseNode
	 * @return
	 */
	private final Class<?> addComposedClassToClassLoader(ClassNode node) {
		String name = node.name.replace('/', '.');

		if (isClassAlreadyLoaded(name)) {
			return returnAlreadyLoadedClass(name);
		}

		ClassWriter temp = new ClassWriter(0);
		node.accept(temp);

		byte[] code = temp.toByteArray();

		org.apache.log4j.Logger.getLogger(VerificationClassLoader.class).debug("add class \"" + name + "\" to classloader for typechecking.");
		
		Class<?> clazz = this.defineComposedClass(name, code, 0, code.length);
		loadedClasses.put(name, clazz);

		return clazz;
	}

	private boolean isClassAlreadyLoaded(String name) {
		return loadedClasses.containsKey(name);
	}

	private Class<?> returnAlreadyLoadedClass(String name) {
		return loadedClasses.get(name);
	}

	@Override
	protected Class<?> findClass(final String name)
			throws ClassNotFoundException {
		org.apache.log4j.Logger.getLogger(VerificationClassLoader.class).debug("find class \"" + name + "\".");
		
		if (isClassAlreadyLoaded(name)) {
			return returnAlreadyLoadedClass(name);
		}

		if (classNodes.keySet().contains(name)) {
			return addComposedClassToClassLoader(classNodes.get(name));
		}

		return super.findClass(name);
	}

}
