import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;

import de.uni_passau.spl.featurebite.adapter.rename.ClassRenameAdapter;
import de.uni_passau.spl.featurebite.adapter.rename.RenameUtil;

/**
 * @author Claus Hunsen
 */
public class RenamingTest {

	static final List<String> CLASSES = new ArrayList<String>();
	static {
		CLASSES.add("examples/rename/Main.class");
		CLASSES.add("examples/rename/Rename.class");
		CLASSES.add("examples/rename/Main$1.class");
		CLASSES.add("examples/rename/Main$2.class");
		CLASSES.add("examples/rename/Main$3.class");
		CLASSES.add("examples/rename/Main$InnerRename.class");
		CLASSES.add("examples/rename/ExceptionRename.class");
		CLASSES.add("examples/rename/RenameAnnotation.class");
	}

	public static final Map<String, String> renameMap = new HashMap<String, String>();
	static {
		renameMap.put("Rename", "Doedel");
		renameMap.put("RenameAnnotation", "DoedelAnnotation");
		renameMap.put("ExceptionRename", "ExceptionDoedel");
	}

	/**
	 * 
	 * @param args
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static void main(String[] args) throws FileNotFoundException, IOException {

		for (String classFile : CLASSES) {

			File source = new File(classFile);

			// read feature class file
			FileInputStream sourceIn = new FileInputStream(source);
			ClassReader cr = new ClassReader(sourceIn);
			ClassWriter cw = new ClassWriter(cr, 0);

			RenameUtil ru = new RenameUtil(renameMap);
			ClassRenameAdapter rca = new ClassRenameAdapter(cw, ru);

			cr.accept(rca, 0);

			sourceIn.close();
			// Files.move(Paths.get(classFile), Paths.get(classFile + ".bak"));
			// source.delete();

			// File target = new File(ru.renameClassFile(classFile));
			File target = new File(classFile);
			FileOutputStream out = new FileOutputStream(target);
			out.write(cw.toByteArray());
			out.close();
		}
	}
}
