package verification;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.asm.tree.ClassNode;

import de.uni_passau.spl.featurebite.util.ClassNodeTransformer;
import de.uni_passau.spl.featurebite.verification.BCELVerifier;
import de.uni_passau.spl.featurebite.verification.VerificationClassLoader;

public class VerifierTest {

	List<ClassNode> classNodes;
	List<File> files;
	private File test1File;
	private ClassNode test1;
	private File test2File;
	private ClassNode test2;
	private File negFile;
	private ClassNode neg;
	private File path;

	private VerificationClassLoader constructClassLoader(ClassNode node1,
			ClassNode... nodes) {
		Set<ClassNode> mySet = new HashSet<ClassNode>();
		Collections.addAll(mySet, node1);
		Collections.addAll(mySet, nodes);
		return new VerificationClassLoader(mySet,
				ClassLoader.getSystemClassLoader());
	}

	@Before
	public void before() throws FileNotFoundException, IOException {
		files = new ArrayList<File>();
		classNodes = new ArrayList<ClassNode>();

		path = new File("testcases/verification/");

		test1File = new File(path, "tmp/Test1.class");
		test1 = ClassNodeTransformer.createClassNodeFrom(test1File);
		files.add(test1File);
		classNodes.add(test1);

		test2File = new File(path, "tmp/Test2.class");
		test2 = ClassNodeTransformer.createClassNodeFrom(test2File);
		files.add(test2File);
		classNodes.add(test2);

		negFile = new File(path, "tmp/Neg.class");
		neg = ClassNodeTransformer.createClassNodeFrom(negFile);
	}

	@Test
	public void testByClassNode() throws IOException {

		VerificationClassLoader loader = constructClassLoader(test1, test2);
		BCELVerifier bcel = new BCELVerifier(classNodes, loader);

		bcel.verify();

		String trace = bcel.getTrace();
		Assert.assertFalse(trace.contains("failed"));
	}

	@Test(expected = NoClassDefFoundError.class)
	public void testByClassNodeFails() throws IOException {

		VerificationClassLoader loader = constructClassLoader(neg);
		BCELVerifier bcel = new BCELVerifier(neg, loader);

		bcel.verify();
		
		

		String trace = bcel.getTrace();
		Assert.assertTrue(trace.contains("failed"));
	}

	@Test(expected = NoClassDefFoundError.class)
	public void testByClassNodeFails2() throws IOException {

		VerificationClassLoader loader = constructClassLoader(test2);
		BCELVerifier bcel = new BCELVerifier(test2, loader);

		bcel.verify();

		String trace = bcel.getTrace();
		Assert.assertTrue(trace.contains("failed"));
	}
	
}
